/*
	<libnet.c>>
ңУǺץ
	 ꥨ ͥåȥңУǥ <ͥåȥط饤֥>
	 Version 0.001, October 2000
    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
	 ------------------------------------------------------------
	 Copyright (C) 2000,2001,2002,2003ǯ naoki kuwamori<naoki@kuwamori.com>

	 ܥץϥե꡼եȥǤʤϡFree Software Foundation
	 ɽGNU ̸ͭѵΡ֥С󣲡װϤʹߤγƥС
	 椫餤줫򤷡ΥС˽äܥץ
	 ۤޤѹ뤳ȤǤޤ

	ʳΥ饤󥹤˾ξϡ᡼뤯.
	<naoki@kuwamori.com>


	Laurier Net work RPG server<game network library>
	Version 0.001, October 2000
    $Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $
	Copyright (C) 2000,2001,2002,2003 Naoki Kuwamori <naoki@kuwamori.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or any
	later version.

	If you wish another license . 
	mail me <naoki@kuwamori.com> .

/etc/servicese game_rpgrpg_chatrpg_clientΥȥɬפ롣
*/
static char cvs_id[] = "$Id: libnet.c,v 0.41 2003/12/02 03:04:10 naohki Exp $";


/*
Υ饤֥礭ä

ͥåȥ̿

ͭ

ʬޤ
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include "libnet.h"

static int * rpg_FDS = NULL;
static int * rpg_IDS = NULL;
static int * rpg_IDSR = NULL;
static struct sockaddr_in * rpg_UDPAD = NULL;


struct pollfd *s_Poll = NULL;
static int servfdT = 0;
static int servfdU = 0;
static int servfdUD = 0;
static int rpg_fdnom = 0;
struct rlimit fdlim;
static int flag = 0;
#define  RPGDEBUGLEVEL 1000

void brakepipe(int data){
	rpglog (RPGDEBUGLEVEL ,"libnet:get broken pipe\n");
	/*ïΥʥ뤫Ϥ狼ʤҤȤޤfʤˤ⤷ʤ*/
	/*Υ顼Ǥ롣*/

	return;
 
}	 

void  rpg_childdeth(int data){
int status;
	rpglog(RPGDEBUGLEVEL , "libnet:Is child prosess die?\n");
	while (waitpid (-1,&status,WNOHANG)>0){
		if (signal (SIGCHLD , rpg_childdeth) == SIG_ERR){/*ʥľ*/
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal error\n");
			exit (1);
		}
	}
	return ;
}

void rpg_endproc(int data){
	int cnt;
 
	/*줬ʤȡåȤĤäƤޤʥФ
	äƤ⥽åȤХɤǤʤ*/
	/*Ǥ⤦ޤʤ*/
	rpglog(RPGDEBUGLEVEL , "libnet:Got end signal or got end request \n");
	/*˻ҥץʤɤ˽λΤｪλƱƤ餦Τɤ*/

	/*ƤΣƣĤĤ*/
	for (cnt = 0 ; cnt < rpg_fdnom ; cnt ++){
		if (rpg_FDS[cnt] == 0)continue;
		rpg_closefd (rpg_FDS[cnt]);
	}
	if (rpg_FDS != NULL){free (rpg_FDS);rpg_FDS= NULL;}
	if (s_Poll != NULL){free (s_Poll);s_Poll= NULL;}
	if (rpg_IDS != NULL){free (rpg_IDS);rpg_IDS= NULL;}
	if (rpg_IDSR != NULL){free (rpg_IDSR);rpg_IDSR= NULL;}
	if (rpg_UDPAD != NULL){free (rpg_UDPAD);rpg_UDPAD= NULL;}
	
	RPG_shm_detach();
	rpg_mallclose();
	exit (1);
}
/*
	ƣĤơ֥¸ߤ뤫ǧ롣
*/
static int test_fd(int fd ){
int cnt;

	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
		if (rpg_FDS[cnt] == fd){
			return cnt;
		}
	}
	return LIBNETNOFD;
}

/*ƣĤΥǡ١餢Ƥõ֤*/
static int getfdnewentry(){
int cnt;
	/*ޤƤʤä*/
	clear_fddatabase();
	
	/*ƤȤܤ*/
	for (cnt = 0 ; cnt < rpg_fdnom ; cnt ++ ){
		if (rpg_FDS[cnt] == 0){
			rpg_IDS[cnt] = (int)NULL;
			rpg_IDSR[cnt] = (int)NULL;
    		rpg_UDPAD[cnt].sin_family = (int)NULL;
    		rpg_UDPAD[cnt].sin_port = (int)NULL;

			return cnt;
		}
	}
	/*Ǥ˳ƤȤ̵ȤϿΰ֤*/
	rpg_IDS[rpg_fdnom]= 0;
	rpg_IDSR[rpg_fdnom]=0;
	rpg_UDPAD[cnt].sin_family = (int)NULL;
	rpg_UDPAD[cnt].sin_port = (int)NULL;
	rpg_fdnom++;
	/*ƣĿƥΥߥåȤˤʤäϥ顼֤*/
	if (rpg_fdnom >= fdlim.rlim_cur ){
		return LIBNETCATCREATFD;
	}
	return (rpg_fdnom-1);
}

/*ΣƣĤ򥯥ơǡ١*/
void	rpg_closefd(int FD){
int cnt;

	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
		if (rpg_FDS[cnt] == FD){
			if (FD == servfdT ) servfdT = (int)NULL;
			if (FD == servfdU ) servfdU = (int)NULL;
			if (FD == servfdUD ) servfdUD = (int)NULL;
			close (rpg_FDS[cnt]);
			rpg_FDS[cnt] = (int)NULL;
			rpg_IDS[cnt] = (int)NULL;
			rpg_IDSR[cnt] = (int)NULL;
			rpg_UDPAD[cnt].sin_family = (int)NULL;
			rpg_UDPAD[cnt].sin_port = (int)NULL;
		}
	}
}

/*ƤΣƣĤ򥯥
ơǡ١⥯ꥢ
Ƴݤ*/

void rpg_fd_allclose(){
int cnt;
	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
		if (rpg_FDS[cnt] != 0){
			close (rpg_FDS[cnt]);
			rpg_FDS[cnt] = ( int )NULL;
			rpg_IDS[cnt] = ( int )NULL;
			rpg_IDSR[cnt] = ( int )NULL;
			rpg_UDPAD[cnt].sin_family = (int)NULL;
			rpg_UDPAD[cnt].sin_port = (int)NULL;
		}
	}
	flag = 0;/*ΰƳݤΤΥե饰򥯥ꥢ*/
	/*ݤ줿򥯥ꥢ*/
	if (rpg_FDS != NULL){free (rpg_FDS);rpg_FDS= NULL;}
	if (s_Poll != NULL){free (s_Poll);s_Poll= NULL;}
	if (rpg_IDS != NULL){free (rpg_IDS);rpg_IDS= NULL;}
	if (rpg_IDSR != NULL){free (rpg_IDSR);rpg_IDSR= NULL;}
	if (rpg_UDPAD != NULL){free (rpg_UDPAD);rpg_UDPAD= NULL;}
	/*δؿͤʤΤͤϸʤ*/
	/*clear_fddatabase();*/

}

/*ƤΣƣĤ򥯥
ơǡ١⥯ꥢ
Ƴݤ*/

void rpg_fd_allclosewithout(int FD){
int cnt;
	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
		if (rpg_FDS[cnt] == FD)continue;
		if (rpg_FDS[cnt] != 0){
			close (rpg_FDS[cnt]);
			rpg_FDS[cnt] = ( int )NULL;
			rpg_IDS[cnt] = ( int )NULL;
			rpg_IDSR[cnt] = ( int )NULL;
			rpg_UDPAD[cnt].sin_family = (int)NULL;
			rpg_UDPAD[cnt].sin_port = (int)NULL;
		}
	}
}

/*ǡ١
ǽΰ󤷤򤷤ʤΤǤޤäˤˤ
rpg_fd_allcloseȤɬפ*/

static int clear_fddatabase(){
int cnt;
	
	/*ǽλϣƣĤޤƽ򤹤*/
	if (flag==0){
		rpglog ( RPGDEBUGLEVEL , "libnet:init net lib\n");
		/*ƣĤκ򥷥ƥफåȤ*/
		getrlimit(RLIMIT_NOFILE , & fdlim);
		rpglog ( RPGDEBUGLEVEL , "libnet:max file open [%d]\n" , fdlim.rlim_cur);
		/*ǡ١ݤƽ*/
		if (rpg_FDS == NULL )rpg_FDS = malloc( sizeof (int) * fdlim.rlim_cur );
		if (rpg_FDS == NULL){
			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error [%d]\n" , errno );
			return LIBNETMALOCCERR;
		}
		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
			rpg_FDS[cnt] = 0;
		}
		/*ѤΣǡ١ݤƽ*/
		if (s_Poll == NULL )s_Poll = malloc( sizeof (int) * fdlim.rlim_cur );
		if (s_Poll == NULL){
			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error s_Poll[%d]\n" , errno );
			return LIBNETMALOCCERR;
		}
		bzero (s_Poll , sizeof (int) * fdlim.rlim_cur);
		/*񤭽ФѤΣֹ̤*/
		if (rpg_IDS == NULL )rpg_IDS = malloc( sizeof (int) * fdlim.rlim_cur );
		if (rpg_IDS == NULL){
			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error [%d]\n" , errno );
			return LIBNETMALOCCERR;
		}
		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
			rpg_IDS[cnt] = 0;
		}
		/*ɤ߹ߤ褦Σֹ̤*/
		if (rpg_IDSR == NULL )rpg_IDSR = malloc( sizeof (int) * fdlim.rlim_cur );
		if (rpg_IDSR == NULL){
			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error [%d]\n" , errno );
			return LIBNETMALOCCERR;
		}
		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
			rpg_IDSR[cnt] = 0;
		}
		/*udp б*/
		if (rpg_UDPAD == NULL )rpg_UDPAD = malloc( sizeof (struct sockaddr_in) * fdlim.rlim_cur );
		if (rpg_UDPAD == NULL){
			rpglog ( RPGDEBUGLEVEL , "libnet:malloc error [%d]\n" , errno );
			return LIBNETMALOCCERR;
		}
		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
			rpg_UDPAD[cnt].sin_family = (int)NULL;
			rpg_UDPAD[cnt].sin_port = (int)NULL;
		}
		rpg_fdnom = 0;
		/*ѥפ»ƥХåؿ򥻥åȤ롣*/
		if (signal (SIGPIPE , brakepipe) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
		/*ҥץȤӲΤɤ
			ҥץȤȤϥ(wait3)ư
			Ʊˡ̵¥֥åδ⤢ΤǤϼʤ*/
		if (signal (SIGCHLD , rpg_childdeth) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
		/*λʥ˥åȤ򥯥򤹤롢Ĥεư
		åȤΥХɤ򤹤ˤǤ褦ˡʤʤɤС*/
#if 1
		if (signal (SIGHUP , rpg_endproc) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
		if (signal (SIGTERM , rpg_endproc) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
		if (signal (SIGQUIT , rpg_endproc) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
		if (signal (SIGINT , rpg_endproc) == SIG_ERR){
			rpglog ( RPGDEBUGLEVEL , "libnet:Set signal errror\n");
			exit (1);
		}
#endif
		flag = 1;
		return (LIBNETOK);
	}
	return LIBNETERR;
}

/*ƣĥǡ١˥ݡ󥰤볰ƣĤϿ*/
int rpg_addRawPollFD(int FD){
int ent;
	if (test_fd(FD)!=LIBNETNOFD){
		rpglog ( RPGDEBUGLEVEL , "libnet:The FD aredy seted\n");
		return LIBNETFDAREDYSET;
	}
	ent = getfdnewentry();

	rpg_FDS[ent]=FD;
	rpg_IDS[ent] = -1;
	rpg_IDSR[ent] = -1;

	return LIBNETOK ;
}

/*ФȤƥåȤ򥪡ץ󤹤*/
int rpg_serveropen(int type ,	char * service_name ){

struct servent *serv=NULL;
struct sockaddr_in		saddrt;
struct sockaddr_un		saddru;
int newfdent;

	/*ǽλϣƣĤޤƽ򤹤*/
	rpglog (100 , "rpg_serveropen:open server with servicename[%d][%s]\n" , type , service_name);
	if (flag==0){
		clear_fddatabase();
	}

	if (strlen (service_name) < 3)return LIBNETERR;
	/*åȥפ˥ååȤξ礫ĥå̾Ȥ*/

	if (type == AF_UNIX){
		if (servfdU != 0){
			return LIBNETOREDYSOPEN;
		}
		newfdent = getfdnewentry();
		/*åȤ롣ΥåȤUNIXɥᥤǡȥ꡼෿åȡ*/
		if ((rpg_FDS[newfdent] = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
			rpglog(RPGDEBUGLEVEL,"libnet:make socket with UNIX domain and STREAM type\n" );
			perror("socket");
			return LIBNETMAKESOCERR;
		}
		rpglog(200,"libnet:if dont set 0 in saddr it may error on bind()\n" );

		/*saddrȤ0ˤƤʤȡbind()ǥ顼뤳Ȥ*/
		bzero((char *)&saddru, sizeof(saddru));
		rpglog(RPGDEBUGLEVEL,"libnet:set socket name\n" );


		saddru.sun_family = AF_UNIX;
		strcpy(saddru.sun_path, service_name);
	
		rpglog(RPGDEBUGLEVEL,"libnet:bind socket address\n" );
		/*åȤ˥ɥ쥹Хɤ롣bind()Ԥʤ褦ǽunlink()
		ѤơåȤбեõ롣*/
		unlink(service_name);
		if (bind(rpg_FDS[newfdent], (struct sockaddr *)&saddru,
				 sizeof(saddru.sun_family) + strlen(service_name)) < 0){
			rpglog(RPGDEBUGLEVEL,"libnet:bing error %s\n" , strerror(errno) );
			close (rpg_FDS[newfdent]);
			perror("bind");
			return LIBNETSOCBINDERR;
		}
	
		rpglog(RPGDEBUGLEVEL,"libnet:listen to socket\n" );
		/*listen򥽥åȤФȯԤ*/
		if (listen(rpg_FDS[newfdent], 1) < 0) {
			rpglog(RPGDEBUGLEVEL,"libnet:listen error [%s]\n"	 , strerror(errno) );
			perror("listen");
			return LIBNETLISSTENERR;
		}
		servfdU = rpg_FDS[newfdent];
		return rpg_FDS[newfdent];

	}else if (type == AF_INET){
		int cnt;
		if (servfdT != 0){

			return LIBNETOREDYSOPEN;
		}
		newfdent = getfdnewentry();
		/*TCPåȤξ*/
		/* åȤ̾Ƥ*/
		/*ϣ𥽥åȤѤ롣*/
		saddrt.sin_family = AF_INET;
		saddrt.sin_addr.s_addr = INADDR_ANY;

		for (cnt=0;cnt<strlen(service_name);cnt++){
			int testd;
			testd = service_name[cnt];
			if (!isdigit(testd)){

				/*ӥ̾ݡֹ롣*/
				if ( (serv = getservbyname( service_name , "tcp" ) ) == NULL){
					rpglog (RPGDEBUGLEVEL , "libnet:getservbyname %s error\n" , getservbyname);
					perror ("getservbyname");
					return LIBNETGETPORTERR ;
				}
				saddrt.sin_port = serv->s_port;
				break;
			}else{
				saddrt.sin_port = htons((short)atoi(service_name));
			}
		}

		/*åȤ롣ΥåȤUNIXɥᥤǡȥ꡼෿åȡ*/
		if ((rpg_FDS[newfdent] = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
			rpglog (RPGDEBUGLEVEL , "libnet:socket error : socket make error \n" );
			perror("socket");
			return LIBNETMAKESOCERR;
		}

		/*åȤ˥ɥ쥹Хɤ롣*/
		if (bind(rpg_FDS[newfdent], (struct sockaddr *)&saddrt, sizeof(saddrt)) < 0){
			rpglog ( 200 , "libnet:socket error : socket bind error port [%d]\n" , serv->s_port );
			close (rpg_FDS[newfdent]);
			perror("bind");
			return LIBNETSOCBINDERR;
		}

		/* listen򥽥åȤФȯԤ*/
		if (listen(rpg_FDS[newfdent], 1) < 0) {
			rpglog (RPGDEBUGLEVEL , "libnet:listen socket error\n");
			perror("listen");
			return LIBNETLISSTENERR;
		}
		servfdT = rpg_FDS[newfdent];
		rpglog(RPGDEBUGLEVEL , " libnet:success init \n");
		return rpg_FDS[newfdent];
	}else if(type == LIBNET_UDP) {
		int cnt;
		if (servfdUD!= 0){
			return LIBNETOREDYSOPEN;
		}
		newfdent = getfdnewentry();
		/*TCPåȤξ*/
		/* åȤ̾Ƥ*/
		/*ϣ𥽥åȤѤ롣*/
		saddrt.sin_family = AF_INET;
		saddrt.sin_addr.s_addr = INADDR_ANY;
		for (cnt=0;cnt<strlen(service_name);cnt++){
			int testd;
			testd = service_name[cnt];
			if (!isdigit(testd)){

				/*ӥ̾ݡֹ롣*/
				if ( (serv = getservbyname( service_name , "udp" ) ) == NULL){
					rpglog (RPGDEBUGLEVEL , "libnet:getservbyname %s 顼\n" , getservbyname);
					perror ("getservbyname");
					return LIBNETGETPORTERR ;
				}
				saddrt.sin_port = serv->s_port;
				break;
			}else{
				saddrt.sin_port = htons((short)atoi(service_name));
			}
		}

		/*åȤ롣ΥåȤIPɥᥤǡDGRAM åȡ*/
		if ((rpg_FDS[newfdent] = socket(AF_INET, SOCK_DGRAM , 0)) < 0) {
			rpglog (RPGDEBUGLEVEL , "libnet:socket error : make socket error\n" );
			perror("socket");
			return LIBNETMAKESOCERR;
		}

		/*åȤ˥ɥ쥹Хɤ롣*/
		if (bind(rpg_FDS[newfdent], (struct sockaddr *)&saddrt, sizeof(saddrt)) < 0){
			rpglog ( RPGDEBUGLEVEL , "libnet:socket error : socket bind error port [%d]\n" , serv->s_port );
			close (rpg_FDS[newfdent]);
			perror("bind");
			return LIBNETSOCBINDERR;
		}

		servfdUD = rpg_FDS[newfdent];
		rpglog(RPGDEBUGLEVEL , " libnet:UDP sucsess init\n");
		return rpg_FDS[newfdent];

	}else{
		rpglog(RPGDEBUGLEVEL , " libnet:You miss SOCtype . didnt open server yet[%d]\n" , AF_INET);

	}
	
	return (LIBNETERR);
}

/*饤ȤȤƣϩ򥪡ץ󤹤*/
int rpg_tcp_client_open( char * hostname , char * service_name ){
	int FD;
	struct servent *serv;
	struct hostent *hp;
	struct sockaddr_in	  addr;
	int cnt;

	if (flag==0){
		clear_fddatabase();
	}
	/*åȤκ*/
	if ((FD = socket(AF_INET, SOCK_STREAM, 0)) < 0) {	
		rpglog (RPGDEBUGLEVEL , "libnet:error make socket\n");
		perror("socket");
		return LIBNETMAKESOCERR;
	}

	 /* addrȤ0ˤƤʤȡbind()ǥ顼뤳Ȥ*/
	bzero((char *)&addr, sizeof(addr));


	 /* åȤ̾Ƥ*/
	if ((hp = gethostbyname(hostname)) == NULL) {
		rpglog (RPGDEBUGLEVEL , "libnet:miss socket name \n");
		perror("No such host");
		return LIBNETSETHOSTNEMEERR;
	}
	bcopy(hp->h_addr, &addr.sin_addr, hp->h_length);
	addr.sin_family = AF_INET;

	/*ӥ̾ݡֹ롣*/
	for (cnt=0;cnt<strlen(service_name);cnt++){
		int testd;
		testd = service_name[cnt];
		if (!isdigit(testd)){

			/*ӥ̾ݡֹ롣*/
			if ( (serv = getservbyname( service_name , "tcp" ) ) == NULL){
				rpglog (RPGDEBUGLEVEL , "libnet:libnet:getservbyname %s error\n" , getservbyname);
				perror ("getservbyname");
				return LIBNETGETPORTERR ;
			}
			addr.sin_port = serv->s_port;
			rpglog (RPGDEBUGLEVEL ,"libnet:param port[%d]\n" , serv->s_port);
			break;
		}else{
			addr.sin_port = htons((short)atoi(service_name));
		}
	}


	/*СȤ³ߤ롣*/
	if (connect(FD, (struct sockaddr *)&addr, sizeof(addr)) < 0){
		rpglog (RPGDEBUGLEVEL , "libnet:Try open socket fault [%d]\n" , errno);
		close(FD);
		perror("connect");
		return LIBNETCONECTERR;
	}
	/*ңУǤȤƤ̿򤷤¾Υץफ³ǤʤȤΤ餻*/
	if (write (FD , LIBNETASSEPT , strlen(LIBNETASSEPT))< 0){
		close (FD);
		return LIBNETCONECTERR;
	}
	/*ץ󤷤ƣĤȤ*/
	rpg_FDS[getfdnewentry()] = FD;
	
	return FD;
}

int rpg_udp_client_open( char * hostname , char * service_name ){
struct servent *serv;
unsigned short port = 0;
int cnt;

/*ӥ̾ݡֹ롣*/
	for (cnt=0;cnt<strlen(service_name);cnt++){
		int testd;
		testd = service_name[cnt];
		if (!isdigit(testd)){

			/*ӥ̾ݡֹ롣*/
			if ( (serv = getservbyname( service_name , "tcp" ) ) == NULL){
				rpglog (RPGDEBUGLEVEL , "libnet:getservbyname %s error\n" , getservbyname);
				perror ("getservbyname");
				return LIBNETGETPORTERR ;
			}
			port = serv->s_port;
			rpglog (RPGDEBUGLEVEL ,"param port[%d]\n" , serv->s_port);
			break;
		}else{
			port = htons((short)atoi(service_name));
		}
	}

return rpg_udp_client_open_port ( hostname , ntohs(port));

}
int rpg_udp_client_open_port( char * hostname , int port ){
	int FD , tmp;
	struct hostent *hp;
	struct sockaddr_in	  addr;

	if (flag==0){
		clear_fddatabase();
	}
	/*åȤκ*/
	if ((FD = socket(AF_INET, SOCK_DGRAM , 0)) < 0) {	
		rpglog (RPGDEBUGLEVEL , "libnet:miss make socket\n");
		perror("socket");
		return LIBNETMAKESOCERR;
	}

	 /* addrȤ0ˤƤʤȡbind()ǥ顼뤳Ȥ*/
	bzero((char *)&addr, sizeof(addr));

	 /* åȤ̾Ƥ*/
	if ((hp = gethostbyname(hostname)) == NULL) {
		rpglog (RPGDEBUGLEVEL , "libnet:input socket name miss\n");
		perror("No such host");
		return LIBNETSETHOSTNEMEERR;
	}

	/*ץ󤷤ƣĤȤ*/
	tmp=getfdnewentry();
	rpg_FDS[tmp] = FD;
	bcopy(hp->h_addr, &rpg_UDPAD[tmp].sin_addr, hp->h_length);
	rpg_UDPAD[tmp].sin_family = AF_INET;
	rpg_UDPAD[tmp].sin_port = htons(port);
	
	return FD;
}
int rpg_tcp_client_open_port( char * hostname , int port ){
	int FD;
/*	struct servent *serv;*/
	struct hostent *hp;
	struct sockaddr_in	  addr;

	if (flag==0){
		clear_fddatabase();
	}
	/*åȤκ*/
	if ((FD = socket(AF_INET, SOCK_STREAM, 0)) < 0) {	
		rpglog (RPGDEBUGLEVEL , "libnet:miss make socket\n");
		perror("socket");
		return LIBNETMAKESOCERR;
	}

	 /* addrȤ0ˤƤʤȡbind()ǥ顼뤳Ȥ*/
	bzero((char *)&addr, sizeof(addr));

	 /* åȤ̾Ƥ*/
	if ((hp = gethostbyname(hostname)) == NULL) {
		rpglog (RPGDEBUGLEVEL , "libnet:miss input socket name\n");
		perror("No such host");
		return LIBNETSETHOSTNEMEERR;
	}
	bcopy(hp->h_addr, &addr.sin_addr, hp->h_length);
	addr.sin_family = AF_INET;
	addr.sin_port = htons (port);
	rpglog (RPGDEBUGLEVEL ,"param port[%d]\n" , port);


	/*СȤ³ߤ롣*/
	if (connect(FD, (struct sockaddr *)&addr, sizeof(addr)) < 0){
		rpglog (RPGDEBUGLEVEL , "libnet:try server connection [%d]\n" , errno);
		close(FD);
		perror("connect");
		return LIBNETCONECTERR;
	}
	/*ңУǤȤƤ̿򤷤¾Υץफ³ǤʤȤΤ餻*/
	if (write (FD , LIBNETASSEPT , strlen(LIBNETASSEPT))< 0){
		close (FD);
		return LIBNETCONECTERR;
	}
	/*ץ󤷤ƣĤȤ*/
	rpg_FDS[getfdnewentry()] = FD;
	
	return FD;
}

/*Υ˥ååȤΥ饤³򤹤*/
int rpg_usoc_client_open(char * socketname){
	int FD;
	struct sockaddr_un	  addr;

	/*ƤΥץλϥǡ١*/
	if (flag==0){
		clear_fddatabase();
	}
	if (strlen(socketname)>MAXPATHLEN){
		rpglog(RPGDEBUGLEVEL,"libnet:sock name over MAXPATHLEN\n");
		return LIBNETMAKESOCERR;
	}
	rpglog(RPGDEBUGLEVEL,"libnet:connect to child process[%s] \n" , socketname);
	/*åȤ롣ΥåȤUNIXɥᥤǡȥ꡼෿åȡ*/
	if ((FD = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
		perror("socket");
		rpglog(RPGDEBUGLEVEL , "libnet:connecttol:socket error [%s] [%s]\n" ,strerror( errno) , socketname );
		return LIBNETMAKESOCERR;
	}
	rpglog(RPGDEBUGLEVEL,"libnet:make socket\n" );

	/*addrȤ0ˤƤʤȡbind()ǥ顼뤳Ȥ*/
	bzero((char *)&addr, sizeof(addr));

	/*åȤ̾Ƥ*/
	rpglog(RPGDEBUGLEVEL,"libnet:input sock name\n" );
	addr.sun_family = AF_UNIX;
	strcpy(addr.sun_path, socketname);

	/*СȤ³ߤ롣*/
	rpglog(RPGDEBUGLEVEL,"libnet:try server open\n" );
	if (connect(FD, (struct sockaddr *)&addr,
			 sizeof(addr.sun_family) + strlen(socketname)) < 0){
		int cnt;
		perror("connect");
		rpglog(RPGDEBUGLEVEL , "libnet:connecttol:connect error [%d] [%s]\n" , errno , socketname );
		sleep (1);
		/*ԤȤϥȥ饤򤫤ƱưƤΤǽޤ⤷ʤ*/
		for (cnt = 0 ; cnt < 5 ; cnt++){
			if (connect(FD, (struct sockaddr *)&addr,
			 sizeof(addr.sun_family) + strlen(socketname)) < 0){
				perror("connect");
				rpglog(RPGDEBUGLEVEL , "libnet:connecttol:connect error [%d] [%s]\n" ,
									errno , socketname );
				sleep(1);
			}else{
				break;
			}
		}
		if (cnt > 4)return (LIBNETCONECTERR);
	}
	rpglog(RPGDEBUGLEVEL,"libnet:sucess conneck to client\n" );

	rpg_FDS[getfdnewentry()] = FD;
	return FD;
}



/*ңУǴĶȤäƤ򤹤*/
int rpg_send_all(char * command ,  char * senddata , int len){
int cnt;

	for (cnt = 1 ; cnt < rpg_fdnom ; cnt ++){
		rpglog(RPGDEBUGLEVEL , "libnet:rpg_send_all: [%d]-[%d]\n" , cnt , rpg_FDS[cnt]);
		if (rpg_FDS[cnt] > 0){
			rpg_send(rpg_FDS[cnt],command,senddata,len);
		}
	}
	return LIBNETNOFD;

}
/*UDPˤ*/
int rpg_send_udp(int FD ,char * command ,  char * senddata , int len){
/*int stat;*/
int cnt;
int ret ;
libnet_send_st send_st;	

	if (FD == 0){
		rpglog(RPGDEBUGLEVEL , "libnet: Need FD nomber\n");
		return LIBNETNOFD;
	}
	if (command == NULL ){
		return LIBNETEFAULT;
	}
	if (senddata == NULL){
		len = 0;
	}
	/*udp ΣƣĤȽꤹ*/
	if ( rpg_UDPAD[test_fd(FD)].sin_port == (int)NULL){
		rpglog(RPGDEBUGLEVEL , "libnet: this FD not for UDP\n");
		return LIBNETEFAULT;
	}
	/*ΣƣĤϿƤ뤫γǧ*/
	if ((cnt = test_fd(FD))< 0)return LIBNETNOFDENT;
	/*ѥåȤȤ*/
	sprintf (send_st.head , LIBNETPHEADER);/*ңУǥѥåȤǤ뤳Ȥ򼨤ʸ򤤤*/
	/*ץΣɣ*/
	send_st.from_pid=htonl(getpid());
	/*ɣġ꤬Ϣ³ǧ뤿*/
	send_st.ID=htonl(rpg_IDS[cnt]++);
	send_st.length=htonl(len);
	send_st.status = 0;/*ơ*/
	memset (send_st.command , 0 , COMMANDLEN);
	memcpy (send_st.command , command , COMMANDLEN);
	/*إåΰ*/
	ret =sendto(FD,  &send_st , sizeof (libnet_send_st), 0, (struct sockaddr *)&rpg_UDPAD[test_fd(FD)], sizeof(rpg_UDPAD[test_fd(FD)]));

	if (ret < 0){
		rpglog(RPGDEBUGLEVEL , "libnet: head fault send message errno[%n]\n" , errno);
		if (errno == EPIPE ){
			rpglog(RPGDEBUGLEVEL , "libnet:got socket close\n");
			rpg_closefd(FD);
		}
		send_st.status = LIBNETSENDERR;
		return LIBNETSENDERR;
	}
	/*ǡΰ*/
	if (senddata != NULL){
		ret =sendto(FD,  senddata , len , 0, (struct sockaddr *)&rpg_UDPAD[test_fd(FD)], sizeof(rpg_UDPAD[test_fd(FD)]));
		if (ret < 0){
			rpglog(RPGDEBUGLEVEL , "libnet:fault send message errno[%n]\n" , errno);
			if (errno == EPIPE ){
				rpglog(RPGDEBUGLEVEL , "libnet:got socket close \n");
				rpg_closefd(FD);
			}
			send_st.status = LIBNETHEADSENDERR;
			return LIBNETHEADSENDERR;
		}
	}
	send_st.status = ret;
	return (LIBNETOK);
}

/*ңУǴĶȤäƤ򤹤*/
int rpg_send(int FD ,char * command ,  char * senddata , int len){
/*int stat;*/
int cnt;
int ret ;
libnet_send_st send_st;	

	if (FD == 0){
		rpglog(RPGDEBUGLEVEL , "libnet:need FD nom tu operate\n");
		return LIBNETNOFD;
	}
	if (command == NULL ){
		return LIBNETEFAULT;
	}
	if (senddata == NULL){
		len = 0;
	}
	/*գģХåȤäգģ*/
    if ( rpg_UDPAD[test_fd(FD)].sin_port != (int)NULL){
		return rpg_send_udp (FD,command,senddata,len);
	}

	/*ΣƣĤϿƤ뤫γǧ*/
	if ((cnt = test_fd(FD))< 0)return LIBNETNOFDENT;
	/*ѥåȤȤ*/
	sprintf (send_st.head , LIBNETPHEADER);/*ңУǥѥåȤǤ뤳Ȥ򼨤ʸ򤤤*/
	send_st.from_pid=htonl(getpid());
	/*ɣġ꤬Ϣ³ǧ뤿*/
	send_st.ID=htonl(rpg_IDS[cnt]++);
	send_st.length=htonl(len);
	send_st.status = 0;/*ơ*/
	
	memset (send_st.command , 0 , COMMANDLEN);
	memcpy (send_st.command , command , COMMANDLEN);
	/*إåΰ*/
	ret = write ( FD , &send_st , sizeof (libnet_send_st));
	if (ret < 0){
		rpglog(RPGDEBUGLEVEL , "libnet:head faulr send errno[%n]\n" , errno);
		if (errno == EPIPE ){
			rpglog(RPGDEBUGLEVEL , "libnet:got socket close \n");
			rpg_closefd(FD);
		}
		send_st.status = LIBNETSENDERR;
		return LIBNETSENDERR;
	}
	/*ǡΰ*/
	if (senddata != NULL){
		ret = write ( FD ,senddata , len);
		if (ret < 0){
			rpglog(RPGDEBUGLEVEL , "libnet:fault send errno[%n]\n" , errno);
			if (errno == EPIPE ){
				rpglog(RPGDEBUGLEVEL , "libnet:got socket close\n");
				rpg_closefd(FD);
			}
			send_st.status = LIBNETHEADSENDERR;
			return LIBNETHEADSENDERR;
		}
	}
	send_st.status = ret;
	return (LIBNETOK);
}


/**********************************

		
				ѥåȥإåɡNRPGHEAD
				ѥåʬ
						  ĹãʤϣƣĤФơ
						ǽ䤤碌
						ǽǤʤȤϥ顼֤
				ѥåȹή
						  ѥåȥإå򻲹ͤ˰ĤΥѥåȤ
						礭ȽǤơ꥿󤹤롢
						  ݡ󥰻ϥݡ󥰤餺ˡ
						ѥåȽ¹Ԥʣإå꥿
				ѥåȴ
						ѥåȤˤɣĤޤޤƤ롢Τᡣ
						ΣɣĤϢ³ƤмϢ³Ƥȹͤ뤬
						ä˽ϹԤʤڤͤˤϥѥåȴ֤
						饤֥ǤνϤʤ
						饤¦ȴ¿ѥå¿
						ͥåȥ֤ȤͤΤǡ
						֤κɹ䡢˥󥰤Ρ³Υȥ饤ʤɤͤ
						ɬפХ饤֥경롣
				ץȥ
						  ѥåȥإåΥɤʤȤ
						ʥѥåȤȽǤʤΤ褦ʼ
						ץȥξϤβμ˴롣
						󥰵ڤӡͤˤΤ򤹤롣
				Хåեꥢ­
						  ˼Хåե礭ʤΤ줿Ȥ
						顼֤ơХåեʬƳǼ
						Ĥʬ˴롣
						  ѥåȤϢ³ƤȻפ֤ϡν
						ԤХåեϤ߽Фʬ˴롣
						ʣإå꥿
						󥰵ڤͤˤΤԤ

*************************************/
/*ǡ׵᥵ʬɤ߹*/
static int read_packet(int FD	 , int requestlen , int *read_len , char * buf , int bufsizei , int timeout){




	struct pollfd pollFD;		/*ݡ󥰤Τ˻Ȥ¤*/
	int stat;					/*ͤ򸫤뤿*/
	int ret;					/**/
	int lessofbuf = 0;

	if (requestlen==0){
		*read_len=0;
		return LIBNETOK;
	}
	stat = LIBNETOK;
	*read_len = 0;
	rpglog(RPGDEBUGLEVEL , "libnet:main loop\n");
	if (timeout <= 0)timeout= 1000;

	if (requestlen > bufsizei){
		lessofbuf = requestlen - bufsizei;
		requestlen = bufsizei;
		stat = LIBNETLOSTDATA;
	}

	pollFD.fd = FD;
	pollFD.events = (POLLIN | POLLPRI );
	/*٥ȥ롼*/

	if ((ret = poll (&pollFD , 1 , timeout))< 0){
			rpglog(RPGDEBUGLEVEL , " pollerror [%d]\n" , errno);
		if (errno == ENOMEM)stat = LIBNETMALOCCERR;
		if (errno == EFAULT)stat = LIBNETEFAULT;
		if (errno == EINTR )stat = LIBNETWTIMEUP;
		return (stat);
	} 
	if ((pollFD.revents & (0x0001 | 0x0002))== 0){
		/*ɤ߹߼Ԥʤ*/
		/*ɤ߹߼Իϥ꥿󤹤롣*/
		if (pollFD.revents & POLLERR)stat = LIBNETPOLLERR;
		if (pollFD.revents & POLLHUP)stat = LIBNETPOLLHUP;
		if (pollFD.revents & POLLNVAL)stat = LIBNETPOLLNVAL;
		/*ǡʤȤϥǡڤǵ*/
		return (stat);
	}
	/*ɤ߹ߤγ*/
	ret = read (FD , &buf[*read_len] , requestlen - *read_len);
	if (ret< 0){
		stat = LIBNETBASEERR - errno;
		return (stat);
	} 
	/*Хȼ*/
	if (ret == 0){
		rpg_closefd (FD);
		return LIBNETLENGTHZERO;
	}
	/*ǡʬäϥ꥿󤹤*/
	if ((*read_len = *read_len + ret)>=requestlen){
		if (lessofbuf> 0){
			char * tmpbuf;
			tmpbuf = malloc (lessofbuf);
			if (tmpbuf == NULL ){
				return stat;
			}
			if ((ret = poll (&pollFD , 1 , timeout))< 0){
				rpglog(RPGDEBUGLEVEL , "libnet: pollerror [%d]\n" , errno);
				free (tmpbuf);
				return (stat);
			} 
			if ((pollFD.revents & (0x0001 | 0x0002))== 0){
				rpglog(RPGDEBUGLEVEL , "libnet: pollerror [%d]\n" , errno);
				free (tmpbuf);
				return (stat);
			}
			/*ɤ߹ߤγ*/
			ret = read (FD ,tmpbuf ,lessofbuf );
			if (ret< 0){
				rpglog(RPGDEBUGLEVEL , "libnet:read error [%d]\n" , errno);
				free (tmpbuf);
				return (stat);
			}
			free (tmpbuf);
		}
		return (stat);
	}else{
		rpglog(  RPGDEBUGLEVEL  ,"libnet:not enouf data\n");

	}
	/*ǡƤʤ硢ȥ饤ߥॢʬɤ߹ߤ»ܤơ*/
	rpglog(  RPGDEBUGLEVEL  , "libnet:read_packet:if not have data . try to read (retrytime * timeout) \n" );
	return LIBNETERR;
	

}

/*
٥ȥݡ
*/
int RPG_Poll( char * command , char * data , int buf_length , int *length , int	 timeout  , int *resfd){
	
	int cnt;
	int nStat;
	int i = 0;
	int requestlen;
	libnet_send_st packet_head;
	int stat = LIBNETOK;
	int ret ;
	int read_len;
	int new_fd;
	int len;
	int cnt2;

	*length = 0;
	bzero (command,COMMANDLEN);
	for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt ++){
		if (rpg_FDS[cnt] != 0){
			s_Poll[i].events = POLLIN;
			s_Poll[i].fd = rpg_FDS[cnt];
			i++;
		}
	}
	/*٥Ȥμ*/
	nStat = poll( s_Poll, i , timeout );
	if ( nStat > 0 )	/* ٥Ȥȯ */
	{
		 /* ٥Ȥǥѥå */
		for (cnt = 0 ; cnt < fdlim.rlim_cur ; cnt++){
			if (s_Poll[cnt].revents){
				if (s_Poll[cnt].revents > POLLERR){
					rpg_closefd (s_Poll[cnt].fd);
					return LIBNETPOLLHUP;
				}
				*resfd = s_Poll[cnt].fd;
				/*UNIXåȤ³׵*/
				if ((s_Poll[cnt].fd ==	servfdU) && (servfdU != 0)){
					struct sockaddr_un	  caddr;
					rpglog(RPGDEBUGLEVEL , "libnet:got connect ion request \n");
					len = sizeof(caddr);
					new_fd = getfdnewentry();
					if ((rpg_FDS[new_fd] = accept(s_Poll[cnt].fd ,	
						(struct sockaddr *)&caddr, &len)) < 0) {
						perror("accept");
						rpglog(RPGDEBUGLEVEL , "libnet:acept error WHY!![%d]\n" , errno);
						if (rpg_FDS[new_fd] > 0)rpg_closefd(rpg_FDS[new_fd]);
						return LIBNETCONNECTIONERROR;/*exit(1);*/
					}
					/*ƣĤϥǡ˳Ǽ*/
					memcpy (command , LIBNETCONNECTREQ , COMMANDLEN);
					memcpy (data , &rpg_FDS[new_fd] , sizeof (int));
					memcpy (&(data[sizeof (int)]),&caddr,sizeof(caddr));
					*resfd = rpg_FDS[new_fd];
					*length = sizeof (int);
					/*׵ƥץȤ饳ͥȤȤ֤*/
					rpglog (RPGDEBUGLEVEL , "libnet:return that have  connect [%d]-[%d] \n"
					, new_fd , rpg_FDS[new_fd] );
					return (LIBNETCONNECTREQUESTU);
				}
				/*³׵ץץȤơƣĤϿ*/
				if ((s_Poll[cnt].fd ==	servfdT) && (servfdT != 0)){
					struct sockaddr_in	  caddr;
					rpglog (RPGDEBUGLEVEL , "libnet:conection request\n");
					len = sizeof(caddr);
					new_fd = getfdnewentry();
					/*ץȤ򤹤*/
					if ((rpg_FDS[new_fd] = accept(s_Poll[cnt].fd ,	
						(struct sockaddr *)&caddr, &len)) < 0) {
						/*ԤФƼԤ֤*/
						rpglog (RPGDEBUGLEVEL , "libnet:acept error WHY!![%d]\n" , errno);
						if (rpg_FDS[new_fd] > 0)rpg_closefd(rpg_FDS[new_fd]);
						return LIBNETCONNECTIONERROR;
					}
					/*ץ׵᤬ΣţԣңУǤƤ뤫ǧ*/
					if (timeout < 1000 )timeout=1000;
					ret=read_packet(rpg_FDS[new_fd] , 12 , length , data , buf_length , timeout);
					/*³夹ˣΣţԣңУǤΥץ׵Ǥ뤳Ȥ򼨤ѥåȤϤ*/
					if (ret < 0 ){
						/*ǡʤȤϥͥ󥨥顼Ȥƽ*/
						rpglog (RPGDEBUGLEVEL , "libnet:it must have packet[%d]-[%d]\n" , ret , timeout);
						rpg_closefd (rpg_FDS[new_fd]);
						return LIBNETCONNECTIONDATAERR;
					}
					/*Ƥǧ*/
					if (memcmp (data , LIBNETASSEPT , strlen(LIBNETASSEPT))!=0){
						rpglog (RPGDEBUGLEVEL , "libnet:test is is true\n");
						rpg_closefd (rpg_FDS[new_fd]);
						return LIBNETCONNECTIONDATAERR;
					}
					/*³λΤ³ƣĤǡ
					³Ȥ֤*/
					memcpy (data , &rpg_FDS[new_fd] , sizeof (int));
					memcpy (&(data[sizeof (int)]),&caddr,sizeof(caddr));
					*resfd = rpg_FDS[new_fd];
					*length = sizeof (int);
					memcpy (command , LIBNETCONNECTREQ , COMMANDLEN);
					rpglog (RPGDEBUGLEVEL , "libnet:return connect if it was sucsess[%d]-[%d] from[%s]\n"
					, new_fd , rpg_FDS[new_fd],inet_ntoa(caddr.sin_addr));
					return LIBNETCONNECTREQUESTT;
				}
				cnt2 = test_fd(s_Poll[cnt].fd);
				/*rpg_IDSäңףƣļȤƽ*/
				if (rpg_IDS[cnt2]==-1){
					/*RAW data äϤΤޤ޼ʤǤ*/
					*length=0;
					*resfd=s_Poll[cnt].fd;
					memcpy (command , LIBNETRAWDATARECEVE , COMMANDLEN);
					rpglog (RPGDEBUGLEVEL , "libnet:Raw data receve FD%d length%d\n",
						*resfd,*length);
					return LIBNETRAWRESEVE;
				}
				/*ǡξ硢إåᤷƽ*/
				ret = read (s_Poll[cnt].fd , &packet_head , sizeof (libnet_send_st));
				if (ret < 0){
					if (errno == EINTR){
						return LIBNETREADBREAK;
					}
					return (LIBNETBASEERR-errno);
				}
				/*Хȼ*/
				if (ret == 0){
					rpg_closefd(s_Poll[cnt].fd);
					return LIBNETLENGTHZERO;
				}
				/*Ĺإå礭ʤ*/
				if (ret < sizeof (libnet_send_st)){
					/*Ĺإåʬ礭*/
					if (ret > strlen (LIBNETPHEADER)){
						/*ӤƥإåǤϤʤäȤϼ*/
						if (memcmp(LIBNETPHEADER , &packet_head , strlen (LIBNETPHEADER))!=0){
							return LIBNETHEADSENDERR;
						}else{
							/*ӤƱϤɤ߹뤫󥸤Ƥߤ롣*/
							if (read_packet (s_Poll[cnt].fd	 , (sizeof (libnet_send_st)-ret) ,
							&read_len , &((char *)&packet_head)[ret] ,(sizeof (libnet_send_st)-ret),
							timeout)!=LIBNETOK){
								/*ǤäȤϥ顼ǵ*/
								return LIBNETHEADBREAK;
							}
						}
					}else {
						/*Ĺإåʬ꾮*/
						return LIBNETHEADSENDERR;
					}
				}
				/*ĹʬʤإåǤ뤫ȽǤ*/
				if (memcmp(LIBNETPHEADER , &packet_head , strlen (LIBNETPHEADER))!=0){
					/*㤦Ȥϥ顼֤*/
					return LIBNETHEADSENDERR;
				}
				/*׵Ĺ*/
				requestlen = ntohl(packet_head.length);
				if (requestlen < 0){
					/*ͤλϥ顼֤*/
					return LIBNETNOREQLEN;
				}
				/*μν򤹤롣*/
				if (cnt2 < 0)return LIBNETERROTHER;
				
				if (rpg_FDS[cnt2] <= 0){
					
					return LIBNETERROTHER;
				}
				/*ɣĤäȤϥơ˿ä*/
				if (++rpg_IDSR[cnt2] != ntohl(packet_head.ID)){
					stat = LIBNETNOIDCONTINU;
					rpglog (RPGDEBUGLEVEL , "libnet:The ID was fault[%d]-[%d]\n" , 
						rpg_IDSR[cnt2] ,ntohl(packet_head.ID));
					rpg_IDSR[cnt2] = ntohl(packet_head.ID);
				}
				memcpy (command	 ,packet_head.command , COMMANDLEN );
				/*ѥåȤɤ߹*/
				ret = read_packet (s_Poll[cnt].fd  , requestlen , length , data , buf_length , timeout);
				if (ret != LIBNETOK )stat = ret;
				return stat;
			}
		}
	}
	else if ( nStat == 0 )	/* ٥Ȥȯ˥ॢȤ */
	{
		/* 顼 */
		memcpy (command , POLLTIMEUP , COMMANDLEN);
		return LIBNETWTIMEUP;
	}
	else if ( errno == EAGAIN  ) /* ׵ǽʤ */
	{
		return (LIBNETBASEERR-errno);
	}
	else if ( errno == EINTR)
	{
		return LIBNETOTHERSIG;
	}
	if ( nStat <= 0 )	/* Ԥ */
	{
		/* ԤΥåȤ򥯥 */
		return LIBNETERROTHER;

	}
	return LIBNETERROTHER;
}

/*٥ȥ롼ΡδؿϻȤʤʤä*/
int rpg_eventloop( char * data , int * leng	 , struct  timeval timeout ){
/*
char	*		data	ǡɤ߹ХåեؤΥݥ
int		*		leng	ǡХåեĹ
struct	timeval timeout	ॢȻ
*/
/*δؿϻȤʤΤȤʤ*/
int stat;
	stat= LIBNETOK;
	return stat;
}

int rpglogfd = 0;
int logconnect = 0;
char rpgPname[20];
/**********************************************************


ϥ¦֤ƤϥФˤĤʤǴ򤹤
ΥФ餫ͳǻ̾硢ơ٥
ƥλ٤Ǥ롣

饤¦˱Ƥϥե
˥ǥХå롣

ФˤȤäƽʥ顼ˤĤƤϥФؤ
ͤƤǤ礦

int loginit( char * pname );		СȤ³
int rpg_loginit( char * pname  , char * sockname);	ФȤ³å̾
int rpg_dbglvl(int dbglvl);
int rpglog(int DBlev , char * errbuf , ...);СäƤƤ餦
char *  rpg_setlogfile(char * filename){ Ф³ʤȤϤδؿǥե̾Ͽ
int rpg_gamelog();  ˽񤭽Ф 	
int rpg_logclose(){	եΥ

ФΤߤȤΤ
char *  rpg_setlogfile(char * filename){ ץ̾Ͽ
int rpg_gamelog( int dbglvl ,char * msg ); ΥϤ
줿ǥХå٥ȥ٤ƤȤƤ錄
int rpg_receveLogwrite(char * readbuf);	ǡ񤭽Ф
int rpg_dbglvl(int dbglvl);
int rpg_logclose(){	եΥ
ˤäƥ󥰥٥򥻥åȤ롣



饤ȤϤޤ

int loginit( char * pname );  ФȤ³
ޤ
int rpg_loginit( char * pname  , char * sockname);	ФȤ³å̾

ˤơФ³򤹤
饤Ȥξ³˼ԤΤǥե˽񤭹褦ˤʤ
СФ³ԤȤϥե˽
Τȡ

int rpglog(int DBlev , char * errbuf , ...);

ˤƥϤ
⤷եˤȤ䡢ɸϤ˽Ф

int rpg_dbglvl(int dbglvl);

ȤοˤɸϤˤɽ
Ф³Ԥƥե˽񤭽ФȤ
ǥХå٥ʲΤΤ񤭹ޤ롢
ΤǥХå٥夲Ⱦܺ٤ˤʤꡢȽפʤΤ
񤫤ʤʤ롣
	   #define RPGEMERGLEVEL	10	 system is unusable				
	   #define RPGALERTLEVEL	20	 action must be taken immediately 
	   #define RPGCRITLEVEL		30	 critical conditions				
	   #define RPGERRLEVEL		40	 error conditions					
	   #define RPGWARNINGLEVEL	50	 warning conditions				
	   #define RPGNOTICELEVEL	60	 normal but significant condition /
	   #define RPGINFOLEVEL		70	 informational					/
	   #define RPGDEBUGLEVEL	20	 debug-level messages				/
ΥǥեȤռФ褦ˤ
饤֥ϤΥ٥˱äƥ󥰤褦˥ǥ󥰤롣

ܺ٤˽Ф褦ʥ٥ˤȤϡνϤȼ
پ徺θ٤Ǥ롣

ƥ٥˱ƵưѤ٤

**********************************************************/

static char * rpg_logsockname(char * socname){
    static char gsockname[MAXPATHLEN];
    static int flag=0;
    if (flag==0){
        bzero(gsockname,MAXPATHLEN);
        strncpy(gsockname,LIBNETLOGSOCNAME,MAXPATHLEN);
        flag=1;
    }
    if (socname!=NULL) strncpy(gsockname,socname,MAXPATHLEN);
    return gsockname;
}

/*ե̾ꡡե̾*/
char *  rpg_setlogfile(char * filename){
	static int fl=0;
	static char gfln[MAXPATHLEN];
	if (fl==0){
		strncpy (gfln , LOGFILENAME , MAXPATHLEN);
		fl = 1;
	}
	if (filename != NULL){
		strncpy (gfln , filename , MAXPATHLEN);
	}
	return gfln;
}

/* С
 * 饤ȤΡRPGLOGREQ򤦤
 * ΥǡȤϤ
 * ǥХå٥򥻥åȤƥ󥰤*/
int rpg_receveLogwrite(char * readbuf){
	int dbglev;

	dbglev=ntohl(*((int *)readbuf));
	return rpg_gamelog(dbglev, &(readbuf[sizeof(int)]));
}

/* 񤭽ФΥǥХå
 * rpg_setlogfile()ˤƥåȤե̾Ф
 * УɣĤղäƥϤ롣
 * ߤϤΥåڤӥơȵǽ̵*/
int rpg_gamelog(int dbglvl ,char * tmpbuf ){
	
static FILE *FD =NULL;
static int linecnt = 0;
time_t timeb;
struct tm *timebuf;
char errerbufffer[LOGBUFFSIZE];
char filename[LOGBUFFSIZE];

	memset (errerbufffer ,(int) NULL , LOGBUFFSIZE);
	memset (filename , (int)NULL , LOGBUFFSIZE);
	if ( time (&timeb) < 0){
		printf ("cant get time\n");
		perror ("time");
	}else{
		timebuf = localtime(&timeb);
		sprintf (errerbufffer , "%d/%d %d:%d:%d " , timebuf->tm_mon +1 ,
		timebuf->tm_mday , timebuf->tm_hour , timebuf->tm_min , timebuf->tm_sec );
	}
	if (dbglvl <= rpg_dbglvl(0)){
		if (FD == NULL){
			/*ǥХåե뤬Ƥʤг*/
			sprintf (filename ,	 "%s.%d" , rpg_setlogfile(NULL) , getpid());
			FD = fopen (filename , "w+");
			if (FD == NULL ){
				/*եΥץ*/
				write(2,"cant open errlog file\n" , strlen("cant open errlog file\n"));
				perror ("open ");	
				write(2,filename , strlen(filename));
				write(2,errerbufffer , strlen(errerbufffer));
				write(2,tmpbuf , strlen (tmpbuf));	/*ʤΤ
							ɸϤ˽Ф*/
			}else{
				/*Τǥե˽Ф*/
				linecnt = 0;
				strncat (errerbufffer , tmpbuf , LOGBUFFSIZE - strlen(errerbufffer) );
				fprintf ( FD , errerbufffer);
				fflush (FD);
				if (rpg_dbglvl(0)%2)write(2,errerbufffer , strlen(errerbufffer));
				linecnt++;
			}
		}else{
			/*ե˵롣*/
			strncat (errerbufffer , tmpbuf, LOGBUFFSIZE - strlen(errerbufffer));
			fprintf ( FD , errerbufffer);
			fflush (FD);
			if (rpg_dbglvl(0)%2)write ( 2 , errerbufffer , strlen(errerbufffer));
			linecnt++;
			
			if (linecnt > LOGLINEMAX){
				if( fseek (FD , 0 , SEEK_SET)< 0){
					fclose (FD);
					FD = NULL;
				}
				linecnt = 0;
			}
		}
	}
	if (dbglvl == -1){
		fclose (FD);
		FD=NULL;
	}
	return RPG_OK ;
}

/*եΥ*/
int rpg_logclose(){
	rpg_gamelog(-1,"");
	return RPG_OK;
}


/*顼ν򤹤*/
int rpg_loginit( char * pname  , char * sockname){
	int ret = 0;
	if (sockname != ""){
		rpg_logsockname(sockname);

		if ( logconnect == 0){/*⤷Ф³Ƥʤä³򤹤*/
			if ( rpglogfd == 0){
				if ( (ret = rpg_usoc_client_open(sockname)) < 0){
					rpglog (RPGDEBUGLEVEL , "libnet:cant connect log server\n");
					return ret;
				}
				rpglogfd = ret;
			}
			if (pname != NULL){
				memset (rpgPname , 0 , 20);
				memcpy (rpgPname , pname , 10);
			}
			logconnect = 1;
		}else{
			rpglog (RPGDEBUGLEVEL , "libnet:log server already connected \n");
			/*ФΣƣĤ֤*/
			return rpglogfd;
		}
	}else{
		rpglog (RPGDEBUGLEVEL , "libnet:log server sockname not abuveable\n");
	}
	return ret;
}
/*顼ν򤹤*/
int loginit( char * pname ){
	int ret = 0;
	if (rpg_logsockname(NULL)!=""){
		if ( logconnect == 0){/*⤷Ф³Ƥʤä³򤹤*/
			if ( rpglogfd == 0){
				if ( (ret = rpg_usoc_client_open(rpg_logsockname(NULL))) < 0){
					rpglog (RPGDEBUGLEVEL , "libnet:cant connect log server\n");
					return ret;
				}
				rpglogfd = ret;
			}
			if (pname != NULL){
				memset (rpgPname , 0 , 20);
				memcpy (rpgPname , pname , 10);
			}
			logconnect = 1;
		}else{
			rpglog(RPGDEBUGLEVEL , "libnet:log server already connected \n");
			/*ФΣƣĤ֤*/
			return rpglogfd;
		}
	}
	return ret;
}


/*ǥХå٥򥻥åȤƼ*/
int rpg_dbglvl(int dbglvl){
	static int rpg_dbglvl1 = 0;
		if (dbglvl > 0){
		rpg_dbglvl1 = dbglvl;
	}
		return rpg_dbglvl1;
}

/*ν񤭽Ф*/
int rpglog(int DBlev , char * errbuf , ...){
time_t timeb;
int ret , n;
struct tm *timebuf;
char errerbufffer[LOGBUFFSIZE];
char tmpbuf[LOGBUFFSIZE];
char filename[sizeof(int) +LOGBUFFSIZE];
va_list ap;

	/*ХåեΥꥢ򤹤*/
	memset (errerbufffer ,(int) NULL , LOGBUFFSIZE);
	memset (tmpbuf , (int)NULL , LOGBUFFSIZE);
	memset (filename , (int)NULL ,sizeof(int) +LOGBUFFSIZE);
	/*֤Υå*/
	if ( time (&timeb) < 0){
		printf ("libnet:cant got time[%s]\n" , strerror (errno));
	}else{
		/*֤򥻥åȤ*/
		timebuf = localtime(&timeb);
		snprintf (errerbufffer ,LOGBUFFSIZE, "%d/%d %d:%d:%d " , timebuf->tm_mon +1 ,
		timebuf->tm_mday , timebuf->tm_hour , timebuf->tm_min , timebuf->tm_sec );
	}
	/*դ᤹*/
	va_start(ap, errbuf);
	(void) vsnprintf(tmpbuf, LOGBUFFSIZE - strlen(errerbufffer) , errbuf , ap );
	va_end(ap);
	/*ФȤ³򤷤ƤʤȤϡ*/
	if (logconnect == 0){
		if (rpg_dbglvl(0) > DBlev){
			if (rpg_dbglvl(0)%2)printf (errerbufffer);
			if (rpg_dbglvl(0)%2)printf (tmpbuf);
		}
		rpg_gamelog(DBlev , tmpbuf);
		return LIBNETNOTOPENLOG;
	}
	/*ƣĤϿƤ뤫ǧ*/
	if (test_fd(rpglogfd) < 0){
		if ((ret = loginit (rpgPname)) < 0){
			logconnect = 0 ;
			rpg_gamelog(DBlev , tmpbuf);
			loginit( rpgPname);
			return (ret);
		}
	}
	n=1;
	/*ФФ򤹤*/
	*((int *)filename)=htonl(DBlev);
	if (rpglogfd > 0){
#if 0
		strncpy (&filename[sizeof (int)] , errerbufffer , strlen (errerbufffer));
		strcat (&filename[sizeof (int)] , rpgPname);	/*襻ơۡˤʤǽ*/
#endif
		strncpy (&filename[sizeof (int)] , rpgPname,LOGBUFFSIZE-1);	
		strcat (&filename[sizeof (int)] , ":" );
		if (LOGBUFFSIZE-strlen(rpgPname)-1>0){
			strncat (&filename[sizeof (int)] , tmpbuf ,LOGBUFFSIZE-strlen(rpgPname)-1);
		}
		ret = rpg_send(rpglogfd , RPGLOGREQ , filename	, sizeof(int)+strlen (&filename[sizeof (int)]));
		if (ret <0){
			rpg_closefd(rpglogfd);
			rpglogfd=0;
			logconnect=0;
			loginit( rpgPname);
		}
	}else{
		rpg_gamelog(DBlev , tmpbuf);
		logconnect=0;
		loginit( rpgPname);
		return LIBNETNOFD;
	}
	/*ǥХå٥뤬λɸ२顼ˤФ*/
	if (rpg_dbglvl(0)%2)write (2, errerbufffer,strlen (errerbufffer));
	if (rpg_dbglvl(0)%2)write (2,tmpbuf,strlen (tmpbuf));
	return (ret);
}

/**********************************************************
ͭ

ͭѤϺ¦ץ¦ץʬޤ
¦ϰʲΥץȤ
char *	RPG_shmctl_init(shment * shmentry , int entrynom  , int *ctlshmid );
char *	RPG_shmctl_addentry(int shmNO  , int size );
void PRG_shmctl_destroy();


¦ϰʲΥץȤ
RPG_shmunit * RPG_shm_init(int ctlshmid);
RPG_shmunit * RPG_shm_all_attach(int ctlshmid );
char * RPG_shm_attach(int shmNO);
void  RPG_shm_all_detach();


ͭ깽¤
typedef struct {
	int shm_id;			ͭɣġʣ
	int size;			ͭ礭
	char * adder;		ͭꥳȥ¦ݥ

}RPG_shmunit;
ι¤Τˤ뤳ι¤ΤΥ쥳ֹ椬
ΤޤޣңУǶֹͭȤʤ롢¦⤳ֹѤ
å򤹤롣
RPG_shmunit * RPG_shm = NULL;
γѿ饤֥ͭƤ롣
ι¤ΤΥ쥳ɿ
#define LIBNETSHMIDMAX			20			ͭ票ȥ꡼
ȤơͤңУǶֹͭκͤȤʤ
Τ᥷ƥˤꡢ¿ζͭꥨȥȤˤ
ͤѹɬפ롣

ͭϿѹ¤
typedef struct{
	int	shmNO;			ңУǶֹͭ椳ֹϿ
	int	size;			ͭ礭
}shment;
ϿΣңУǶֹͭ礭롣


**********************************************************/


RPG_shmunit *RPG_shm = NULL;
/*ͭطνʥå*/
RPG_shmunit * RPG_shm_init(int ctlshmid){
char * adder;
static int opflag = 0;

	if (opflag == 1){
		return RPG_shm;
	}
	if (ctlshmid ==0)return NULL;
	/*ͭΥå*/
	adder = shmat ( ctlshmid , NULL , (int)NULL);
	if (adder < 0){
		rpglog(RPGDEBUGLEVEL , "libnet:faule attatch shm[%s]\n" , strerror(errno));
		adder = NULL;
		return (RPG_shmunit * )(LIBNETSHMATTACHERR);
		/*ͭؤΥåʤ*/
	}
	(char *)RPG_shm = adder;
	opflag = 1;
	/*ͭꥳȥؤΥݥ󥿤֤*/
	return (RPG_shmunit *)adder;
}

/*ͭνʺ*/
char *	RPG_shmctl_init(shment * shmentry , int entrynom  , int *ctlshmid ){
	char * ret3;
	int ret1;
	int cnt;
	
	static int  open_flag= 0;
	
	if (open_flag > 0){
		return (char *)LIBNETOREDYOPEN;
	}
	/*ͭΰμ*/
	if (RPG_shm == NULL){
		ret1 = shmget (IPC_PRIVATE, LIBNETSHMIDMAX * sizeof(RPG_shmunit) , IPC_CREAT | 0666);
		if (ret1 > 0){
			*ctlshmid = ret1;
			RPG_shm = shmat ( *ctlshmid, NULL , (int)NULL );
			if (RPG_shm < 0){
				rpglog(RPGDEBUGLEVEL , "libnet:rpg_shmget:%s\n" , gettext("falure memory attatch"));
				RPG_shm = NULL;
				return ((char *)LIBNETSHMATTACHERR);
			}
		}
	}
	/*ͭΥꥢ*/
	/*ޤ衪*/
	memset (RPG_shm , (int)NULL , LIBNETSHMIDMAX * sizeof(RPG_shmunit));
	RPG_shm[0].shm_id = *ctlshmid;
	RPG_shm[0].size = LIBNETSHMIDMAX * sizeof(RPG_shmunit);
	RPG_shm[0].adder = (char *)RPG_shm;
	open_flag = 1;
	/*¾ͭκ*/
	for (cnt = 0 ; cnt < entrynom ;cnt++){
		/*ͭΰ򤷤٤ƥǡ*/
		if ((shmentry[cnt].shmNO == (int)NULL) || (shmentry[cnt].size == (int)NULL) || 
		(shmentry[cnt].shmNO > LIBNETSHMIDMAX)){
			continue;
		}
		/**/
		ret3 = RPG_shmctl_addentry( shmentry[cnt].shmNO, shmentry[cnt].size );
		if ((int)(ret3) < (int)NULL){
			char * buf = NULL;
			rpglog (RPGDEBUGLEVEL , "libnet:shmaddentry error [%d] [%s]\n" , ret3 ,(char *) rpg_net_perror((int)ret3 , &buf));
			PRG_shmctl_destroy();
			return (ret3);
		}else{
			memset ((char *)ret3,0,shmentry[cnt].size);
		}
	}
	rpglog (RPGDEBUGLEVEL , "libnet:RPG_shmctl_init end [%d]\n" , RPG_shm);
	return ((char *)RPG_shm);
}
/*ͭΥȥɲáʺ*/
char *	RPG_shmctl_addentry(int shmNO  , int size ){
int ret;
	
	if (shmNO >= LIBNETSHMIDMAX){
		return (char *)LIBNETSHMIDerror;
	}
	if (RPG_shm[shmNO].shm_id != (int)NULL && RPG_shm[shmNO].size != (int)NULL){
		return (char *)LIBNETSHMaredyattach;
	}
	ret = shmget(IPC_PRIVATE, size, IPC_CREAT | 0666);
	if (ret > 0){

		RPG_shm[shmNO].shm_id = ret;
		RPG_shm[shmNO].adder = shmat ( RPG_shm[shmNO].shm_id , NULL , (int)NULL );
		if (RPG_shm[shmNO].adder < 0){
			rpglog(RPGDEBUGLEVEL , "libnet:rpg_shmget:fault attatch shm\n");
			RPG_shm[shmNO].adder = NULL;
			return (char*)(LIBNETSHMATTACHERR);
		}
	}else{
		rpglog(RPGDEBUGLEVEL , "libnet:cant got memory[%d] [%d] [%d] [%s]\n" , ret , size , shmNO , strerror(errno));
		RPG_shm[shmNO].adder = NULL;
		return (char *)(LIBNETSHMGETERR);
	}
	RPG_shm[shmNO].size = size;
	return RPG_shm[shmNO].adder;
}

/*ͭ˴*/
void PRG_shmctl_destroy(){
	int cnt;
	
	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
		if (RPG_shm[cnt].shm_id != (int)NULL && RPG_shm[cnt].size != (int)NULL){
			if (RPG_shm[cnt].adder !=NULL){
				shmdt(RPG_shm[cnt].adder);
				shmctl ( RPG_shm[cnt].shm_id , IPC_RMID , NULL); /*ͭҤŤ*/
				RPG_shm[cnt].shm_id = 0;
				RPG_shm[cnt].size = 0;
				RPG_shm[cnt].adder = NULL;
			}
		}
	}
}

/*ͭΥå򤹤*/

char * RPG_shm_attach(int shmNO){
char * adder;
	if (RPG_shm[shmNO].shm_id== (int)NULL){
		
		return (char *)LIBNETSHMNOSHMID;
	}
	adder = shmat ( RPG_shm[shmNO].shm_id , NULL , (int)NULL);
	if (adder < 0){
		rpglog (RPGDEBUGLEVEL , "libnet:fault atatch shm[%s]\n" , strerror(errno));
		adder = NULL;
		return (char *)(LIBNETSHMATTACHERR);
	}
	return (char *)adder;

}
/*Ƥζͭ˥å*/
RPG_shmunit * RPG_shm_all_attach(int ctlshmid ){
int cnt;
RPG_shmunit * ret;
static RPG_shmunit * ctladder= NULL;
	if (ctlshmid == 0)return ctladder;
	if (ctlshmid == -1)FREE(ctladder);
	if ((void *)(ret = RPG_shm_init(ctlshmid)) < NULL){
		return (RPG_shmunit *)ret;
	}
	if (ctladder != NULL){
		free (ctladder);
	}
	(void *)ctladder = malloc (LIBNETSHMIDMAX * sizeof(RPG_shmunit) );
	if ((void *)ctladder <= NULL){
		return	((RPG_shmunit *)LIBNETSHMMALOCCERR);
	}
	memcpy (ctladder , RPG_shm , LIBNETSHMIDMAX * sizeof(RPG_shmunit) );
	
	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
		if ((RPG_shm[cnt].shm_id == (int)NULL) || (RPG_shm[cnt].size == (int)NULL)){
			ctladder[cnt].adder =	 RPG_shm_attach(cnt);
			if (ctladder[cnt].adder <= (int)NULL){
				ctladder[cnt].adder = NULL;
				ctladder[cnt].shm_id = (int)NULL;
				ctladder[cnt].size = (int)NULL;
			}
		}
	}
	return (RPG_shmunit *)(ctladder);
}

/*ƤΥǥå*/

void  RPG_shm_all_detach(){
	int cnt;
	RPG_shmunit * ctladder;
	rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_all_detach start\n");
	ctladder = RPG_shm_all_attach((int)NULL);
	if (ctladder == NULL)return; 
	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++)
		rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_all_detach[%d][%d]\n" , cnt , ctladder[cnt].adder);
		if (ctladder[cnt].adder != NULL)shmdt(ctladder[cnt].adder);
	if (RPG_shm != NULL)shmdt(RPG_shm);
	RPG_shm = NULL;
	RPG_shm_all_attach (-1);
}

/*ƤΥǥå*/

void  RPG_shm_detach(){
	int cnt;
	RPG_shmunit * ctladder;
	rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_detach start\n");
	ctladder = RPG_shm_init((int)NULL);
	if (ctladder == NULL){
		return;
	}
	for (cnt = 0 ; cnt < LIBNETSHMIDMAX ; cnt++){
		rpglog (RPGDEBUGLEVEL , "libnet:RPG_shm_detach[%d][%d]\n" , cnt , ctladder[cnt].adder);
		if (ctladder[cnt].adder != NULL){
			shmdt(ctladder[cnt].adder);
			ctladder[cnt].adder = NULL;
			ctladder[cnt].shm_id = (int)NULL;
			ctladder[cnt].size = (int)NULL;
		}
	}
}
/**********************************************************
ͭmmapؤ

static 

char * rpg_mopen(char * filepath );

void rpg_memclose (char *filepath );
**********************************************************/
static libnet_mmaps libnet_mmap[MAXMEMNOM];

static int rpgmmatnom = 0;
char * rpg_mopen(char * filepath ){
int nom = -1;
struct stat statbuf;
int ret,cnt ;

	/*ǤϿƤ餽ΣģģţҤ֤*/
	for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
		if (strncmp ( filepath , libnet_mmap[cnt].fpath , MAXPATHLEN)==0){
			rpglog (200 , "libnet:rpg_mopen:alredy opend \n");
			return libnet_mmap[cnt].addr;
		}
	}
	for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
		if (libnet_mmap[cnt].addr <= 0){
			nom=cnt;
			break;
		}
	}
	if (nom == -1 ){
		nom = ++rpgmmatnom;
	}
	/*ͤΤĹեѥӽ*/
	if (strlen (filepath) > MAXPATHLEN){
		rpglog (RPGDEBUGLEVEL , "libnet:rpg_mopen:too long file path\n");
		return NULL;
	}
	/*꡼ĥդե򥪡ץ󤹤*/
	libnet_mmap[nom].FD=open (filepath , O_RDWR );
	if (libnet_mmap[nom].FD ==1 ){
		rpglog (RPGDEBUGLEVEL , "libnet:rpg_mopen:cant open file[%s]\n" , filepath);
		return NULL;
	}
	/*եΥåȤ*/
	ret = fstat (libnet_mmap[nom].FD , &statbuf );
	if (ret < 0){
		rpglog (RPGDEBUGLEVEL , "libnet:rpg_mopen:cant get stat of file[%s]\n" , filepath);
		return NULL;
	}
	libnet_mmap[nom].length=statbuf.st_size;
	/*åȤ򸵤˥ե륵礭ǣͣͣФstatbuf.st_size */
#ifdef _POSIX_MAPPED_FILES
	libnet_mmap[nom].addr = 
		mmap (NULL,statbuf.st_size,PROT_WRITE|PROT_READ,MAP_SHARED,libnet_mmap[nom].FD,0);
#endif
	if (libnet_mmap[nom].addr == (void *)-1){
		rpglog (RPGDEBUGLEVEL , "libnet:rpg_mopen:mmap error errno[%d]\n",errno);
		return NULL;
	}
	return libnet_mmap[nom].addr ;
}

void rpg_mclose(char * filepath){
int ret,cnt ;
	/*Ͽ򸡺Ƥ*/
    for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
        if (strncmp ( filepath , libnet_mmap[cnt].fpath , MAXPATHLEN)==0){
            rpglog (200 , "libnet:rpg_mclose:found entry \n");
#ifdef _POSIX_MAPPED_FILES
            ret = munmap (libnet_mmap[cnt].addr ,libnet_mmap[cnt].length );
			if (ret < 0)rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:munmap error \n");
#endif
			libnet_mmap[cnt].addr=NULL;
			libnet_mmap[cnt].FD=0;
			libnet_mmap[cnt].length=0;
			bzero (libnet_mmap[cnt].fpath , MAXPATHLEN);
			return ;
        }
    }
	/*ϿƤʤХ顼åϤ*/
	rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:cant find map to close\n");
}
int rpg_mlength(char * filepath){
int cnt ;
	/*Ͽ򸡺Ƥ*/
    for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
        if (strncmp ( filepath , libnet_mmap[cnt].fpath , MAXPATHLEN)==0){
            rpglog (200 , "libnet:rpg_mlength:found entry \n");
			return libnet_mmap[cnt].length;
        }
    }
	/*ϿƤʤХ顼åϤ*/
	rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:cant find map to close\n");
	return RPG_NG;
}

void rpg_mallclose(){
int ret,cnt ;

    for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
        if (strlen (libnet_mmap[cnt].fpath)>0 ){
            rpglog (200 , "libnet:rpg_mclose:found entry \n");
#ifdef _POSIX_MAPPED_FILES
            ret = munmap (libnet_mmap[cnt].addr ,libnet_mmap[cnt].length );
			if (ret < 0)rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:munmap error \n");
#endif
			libnet_mmap[cnt].addr=NULL;
			libnet_mmap[cnt].FD=0;
			libnet_mmap[cnt].length=0;
			bzero (libnet_mmap[cnt].fpath , MAXPATHLEN);
        }
    }

}

void rpg_msync(char * filepath){
int ret,cnt ;
    for (cnt = 0 ; cnt < rpgmmatnom;cnt++){
        if (strncmp ( filepath , libnet_mmap[cnt].fpath , MAXPATHLEN)==0){
            rpglog (200 , "libnet:rpg_msync:found entry \n");
#ifdef _POSIX_MAPPED_FILES
#ifdef _POSIX_SYNCHRONIZED_IO
            ret = msync (libnet_mmap[cnt].addr ,libnet_mmap[cnt].length  , MS_ASYNC);
			if (ret < 0)rpglog (RPGDEBUGLEVEL , "libnet:rpg_mclose:munmap error \n");
#endif
#endif

			return ;
        }
    }
	/*ϿƤʤХ顼åϤ*/
	rpglog (RPGDEBUGLEVEL , "libnet:rpg_msync:cant find map to close\n");
}

/**********************************************************
ơط

Ǥνϸ§ȤƣģϣӹФнȤ

ư
ƥνλ

#define RPGSECLOGSAFE1		0x000001	ե򤤤ĤХååפ/
#define RPGSECLOGSAFE2		0x000002	ե뤬񤭽ФʤȤϥƥȤ/
#define RPGSECLOGSAFE3		0x000004	äѤˤʤä᡼뤷ä/
#define RPGSEC

#define RPGSECLOGING1		0x000010	ơ٥Υͭˤ/
#define RPGSECLOGING2		0x000020	ơ٥Υʳӽ/
#define RPGSEC
#define RPGSEC

#define RPGSEC				0x000100	/
#define RPGSEC
#define RPGSEC
#define RPGSEC

#define RPGSECNOMCONECT		0x001000	ͥ¤롣/

#define RPGSECWORNINNGDOWN	0x800000	˥󥰥٥ǥƥȤ/

**********************************************************/
/**/
int rpg_seclevel( int level1){
	static int level= 0;
	if (level1 > 0)level = level1;
	if (level1 < 0 ) level= 0;
	return level;
}

/**/
int rpg_secdisksp(int alarm_size){
	int size =0;
	
	
	
	return size;
}

/**/
int rpg_secmailtoadmin(){
	
	
	
	return (LIBNETOK);
}

/**/
int rpg_secbeepmachine(){
	
return RPG_OK;	
}

int rpg_secallwerninng(){
	
	
return RPG_OK;	
}

/**/
int rpg_secall_shutdown(){
	
	
	
return RPG_OK;	
}

/**/
int rpg_savedata(){/*ߤΥǡ򥻡֤*/
	
	
	
return RPG_OK;	
}

int rpg_gethash ( unsigned short * data , int len){
	int cnt;
	int ret = 0;
	for (cnt = 0 ; cnt < len ; cnt++){
		ret = ret * (int)data[cnt];
	}
	return ret;
}


/**********************************************************

		顼åط

**********************************************************/
typedef struct{
	int		rpg_errno;
	char	title[256];
	char	message[1024]; 
}errtable;

errtable errmessage[MAXERRNOM] = {
/*顼ֹ*/
		{20001 ,

/*ȥ*/ 
		"memory aredy open", 
		/*꤬˥ץ󤵤Ƥ*/

/*н*/
	"The memory was aredy open on same ID \n Dont warry you dont have to do any thing.i\nthis is worninng." } 	,
	/*Ǥ˶꤬ͭץ󤵤Ƥ*/

{  10009,"Got connection request",""},		/*³׵*/
{  10008,"Got connection request",""},		/*³׵*/
{  10007,"Open FD",""},		/*ƣĤ򥪡ץ󤷤*/
{  10006,"FD was not open",""},		/*ץ󤵤Ƥʤ*/
{  10005,"Got signal befor wait event",""},		/*׵ᤵ줿٥ȤΤɤ줫˥ʥ뤬ȯ*/
{  10004,"Lost part of data",""},		/*ʬŪ˥ǡ򼺤ä*/
{  10003,"ID isn't continued",""},		/*ɣĤϢ³Ƥʤ*/
{  10002,"Data buffer shorter than input data",""},		/*ǡХåեξ꾮*/
{  10001,"",""},		/**/
{ -10001,"FD was not enterd",""},		/*ƣĤäƤʤ*/
{ -10002,"It was no FD entry",""},		/*ƣĤΥȥ꤬ʤ*/
{ -10003,"",""},		/**/
{ -10004,"header cant read compleate",""},		/*إåɤ߹ޤʤä*/
{ -10005,"cant alloc memory",""},		/*γݼ*/
{ -10006,"falt make socket",""},		/*åȤκ顼*/
{ -10007,"port nom get error",""},		/*ݡֹμ顼*/
{ -10008,"host name error",""},		/*ۥȥ͡Υåȼ*/
{ -10009,"connect error",""},		/*ͥȥ顼*/
{ -10010,"signal set error",""},		/*ʥΥåȥ顼*/
{ -10011,"Error condition",""},		/* Error condition */
{ -10012,"Hung up",""},		/* Hung up */
{ -10013,"Invalid request: fd not open",""},		/* Invalid request: fd not open */
{ -10015,"Invalid argument",""},		/*Ȥƻꤷ󤬡ƤӽФץΥɥ쥹֤˴ޤޤƤʤ*/
{ -10014,"Other error",""},		/*¾Υ顼*/
{ -10015,"Broken header",""},		/*إåʬƤ*/
{ -10016,"read break with signal",""},		/*ɤ߹ߤΥʥˤ*/
{ -10017,"Invalid request len",""},		/*׵Ĺޤϥޥʥ*/
{ -10018,"break with Invalid signal",""},		/* ͽʤʥˤ*/
{ -10019,"server aredy opened",""},		/*Ǥ˥Хץ󤷤Ƥ*/
{ -10020,"Zero read len",""},		/*Ĺξ*/
{ -10021,"falt to connection",""},		/*³μ*/
{ -10022,"cant read connection stringth",""},		/*³³ʸǤʤä*/
{ -10023,"Full of system max FD nom",""},		/*ƥ¤ǤʾƣĤ򥪡ץǤʤ*/
{ -10024,"read length to short",""},		/*ɤ߹Ĺ*/
{ -10100,"Invalid error",""},		/*¾ͽʤ顼*/
{ -20001,"aredy opend ID",""},		/*Ǥ˥ץ󤵤줿ɣ*/
{ -20002,"aredy opend ID",""},		/*Ǥ˥ץ󤵤줿ɣ*/
{ -20003,"memory attatch error",""},		/*ͭꥢå顼*/
{ -20004,"memory get error",""},		/*ͭꥲåȥ顼*/
{ -20005,"no memory ID entry",""},		/*ͭɣĤϿƤޤ*/
{ -20006,"falt malloc",""},		/*Υޥå˼Ԥޤ*/
{ -20007,"no memory ID entry",""},		/*ͭɣĤϿƤޤ*/
{ -30001,"",""},
{0,"",""}
};


/*
 *	CSVɤ߹ߴؿ
 *	yamakawa yamakawa@users.sourceforge.jp
 * Copyright (C) 2002
 * */
#if 0
/* CSVɤ߹ */

typedef struct {
    int iOffset;        /* ¤ΤγФؤΥեå(ñ̥Х) */
    void* (*pfConv)( void* pMem, char* pcCSV, int iParam ); /* CSV -> Ѵؿ */
    int iParam;         /* pfConvؿ裳 */
    int iRepeat;        /* ֤ */
    
} CCSVTemplate;
#define CSVTRECORD( iOffset, pfFunc, iParam ) { iOffset, pfFunc, iParam, 1 }
#define CSVTRECORD_ARRAY( iOffset, pfFunc, iParam, iRepeat ) { iOffset, pfFunc, iParam, iRepeat }
#define EndOfCSVTemplate { 0, NULL, 0, 0 }
#define IsEndOfCSVTemplate( p ) ( (p)->pfConv == (void*(*)(void*,char*,int))NULL )

/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣 */
/* »:CSV1ܤ1024ХȰʲǤ뤳 */
extern int/**/ GetCSV(
    void* pMem,
    char* pcCSV,
    CCSVTemplate* pTemplate
);  
    
/* ƥץ졼Ȥǻꤵ줿CSVΥǡߤ򤹤롣(ƥݥХåեʬѰդ) */
extern int/**/ GetCSV_tmpbuff(
    void* pMem,
    char* pcCSV,
    CCSVTemplate* pTemplate,
    char* pcTmp,    /* CSVɤ߹߻˻Ѥ */
    int sizeTmp
);

/* CSV -> Ѵؿ(CCSVTemplate¤ΤΡpfConv˻ꤹؿ) */
/* ̤δؿͤȤƼΤ褦ʤΤꤵƤ롣 */
/* : */
/*   pMem: */
/*   pCSV:CSV */
/*   iParam:ƴؿͭǽʥѥ᡼ */
/*  */
/*   Υ(iRepeatѻΤ) */
/*   NULLξ硢 */
extern void* CSV_discard( void* pMem, char* pcCSV, int iParam/**/ );  /* CSVʸΤƤ */
extern void* CSV_int( void* pMem, char* pcCSV, int iParam/**/ );      /* CSVʸintѿȤƼ>
 */
extern void* CSV_short( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸshortѿȤ>
 */
extern void* CSV_char( void* pMem, char* pcCSV, int iParam/**/ );     /* CSVʸcharѿȤƼ
 */
extern void* CSV_float( void* pMem, char* pcCSV, int iParam/**/ );    /* CSVʸfloatѿȤ>
 */
extern void* CSV_double( void* pMem, char* pcCSV, int iParam/**/ );   /* CSVʸdoubleѿȤ
 */
extern void* CSV_string( void* pMem, char* pcCSV, int iParam/*'\0'ޤʸ*/ ); /* CSVʸʸ
˥ԡ(ü'\0') */
extern void* CSV_memory( void* pMem, char* pcCSV, int iParam/*Хȿ*/ ); /* Τޤޥԡ */
/* ...ޥؿ򼫺ơѤ뤳Ȥǽ (^^)v */

/*Ȥ

Ȥ
typedef struct{
	short nomber1;
	char somestring[30];
}testst;
Τ褦ʹ¤Τä
---------------file start------------------
125,some nomber name
345,string some thing
--------------file end--------------------
ʤդ뤬ä
ƥץ졼Ȥ
static CCSVTemplate tmpl_templateofcsvrec[] = { 
	CSVTRECORD( offsetof( testst, nomber1),CSV_short,0), 
	CSVTRECORD( offsetof( testst,somestring),CSV_string,30),
	EndOfCSVTemplate
};
ä
testst testval;
ե򳫤Ƥ 
ɤ߹ߤʤ饻åȤޤ
while ( fgets( buff, sizeof( buff ), fp ), !feof( fp ) ) {
	memset (&testval , 0 ,sizeof(testst));
	if (FALSE==GetCSV( &testval, buff, tmpl_templateofcsvrec )){
		printf("error of read cvs file\n");
	}   
	printf("print nom[%d] string[%s]\n", testval.nomber1 , testval.somestring);
}
ʴǻȤޤ
*/
#endif

char *rpg_net_perror(int l_errno ,char ** message){
char *ret = NULL;
int cnt;

	for (cnt = 0 ; errmessage[cnt].rpg_errno != 0 ;cnt++){
		if (errmessage[cnt].rpg_errno == l_errno){
			ret=errmessage[cnt].title;
			if (message !=NULL){
				*message=errmessage[cnt].message;
			}
			break;
		}
	}
	return ret;
}

static char* EscapeCSV( char* pcTmp, char* pcCSV, int sizeTmp )
{
    int count = 0;

    while ( *pcCSV != '\0' && *pcCSV != ',' ) {

        if ( count >= sizeTmp ) {
            /* Ѱդ줿ƥݥХåե礭򤳤硢Ԥ */
            return NULL;
        }
        /* '\'(ʸ) */
        if ( *pcCSV == '\\' ) {
            /* '\'μʸ롣 */
            *pcTmp++ = *( pcCSV + 1 );
            pcCSV += 2;
        }
        else {
            /* ̾ʸϤΤޤޥԡ롣 */
            *pcTmp++ = *pcCSV++;
        }
        ++count;
    }
    *pcTmp = '\0';

    return ++pcCSV;
}

/*
 *  * ãӣʸ󤫤鹽¤Τ˥쥳ɤ򥻥åȤ */
int GetCSV( void* pMem, char* pcCSV, CCSVTemplate* pTemplate )
{
    char pcTmp[ 1024 ];

    return GetCSV_tmpbuff( pMem, pcCSV, pTemplate, pcTmp, sizeof( pcTmp ) );
}

/*
 *  *  */
int GetCSV_tmpbuff(
    void*           pMem,
    char*           pcCSV,
    CCSVTemplate*   pTemplate,
    char*           pcCSVTmp,
    int             sizeCSVTmp )
{
    while ( !IsEndOfCSVTemplate(pTemplate) ) {

        int     i;
        char*   pMemTmp = (char*)pMem + pTemplate->iOffset;

        for ( i = 0 ; i < pTemplate->iRepeat ; ++i ) {
            pcCSV = EscapeCSV( pcCSVTmp, pcCSV, sizeCSVTmp );
            if ( !pcCSV ) {
                return FALSE;
            }
            pMemTmp = pTemplate->pfConv( (char*)pMemTmp, pcCSVTmp,
                pTemplate->iParam );
            if ( !pMemTmp ) {
                return FALSE;
            }
        }
        ++pTemplate;
    }

    return TRUE;
}

/*
 *  * ɤФȤ˻Ѥޤ */
void* CSV_discard(
    void*   pMem/*ߡ*/,
    char*   pcCSV/*ߡ*/,
    int     iParam/*ߡ*/
)
{
    return pMem;
}

/*
 *  * ѴСintegerλ˻Ѥޤ */
void* CSV_int( void* pMem, char* pcCSV, int iParam/**/ )
{

    *(int*)pMem = atoi( pcCSV );
    return (int*)pMem + 1;
}

void* CSV_uint( void* pMem, char* pcCSV, int iParam/**/ )
{

    *(int*)pMem = atoi( pcCSV );
    return (unsigned int*)pMem + 1;
}

/*
 *  * ѴСintegerλ˻Ѥޤ */
void* CSV_short( void* pMem, char* pcCSV, int iParam/**/ )
{

    *(short*)pMem = atoi( pcCSV );
    return (short*)pMem + 1;
}

void* CSV_ushort( void* pMem, char* pcCSV, int iParam/**/ )
{

    *(short*)pMem = atoi( pcCSV );
    return (unsigned short*)pMem + 1;
}
/*
 *  * ѴСcharλ˻Ѥޤ */
void* CSV_char( void* pMem, char* pcCSV, int iParam/**/ )
{
    *(char*)pMem = atoi( pcCSV );
    return (char*)pMem + 1;
}

void* CSV_uchar( void* pMem, char* pcCSV, int iParam/**/ )
{
    *(char*)pMem = atoi( pcCSV );
    return (unsigned char*)pMem + 1;
}

/*
 *  * ѴСʸ(NULLȥå)λ˻Ѥޤ */
void* CSV_string( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
{
    strncpy( pMem, pcCSV, iParam );
    return (char*)pMem + iParam;
}

/*
 *  * ѴСʸ(NULLȥåפʤ)λ˻Ѥޤ */
void* CSV_memory( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
{
    memcpy( pMem, pcCSV, iParam );
    return (char*)pMem + iParam;
}

/*
 *  * ѴСfloatλ˻Ѥޤ */
void* CSV_float( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
{
    *(float*)pMem = (float)atof( pcCSV );
    return (float*)pMem + 1;
}

/*
 *  * ѴСdoubleλ˻Ѥޤ */
void* CSV_double( void* pMem, char* pcCSV, int iParam/*ʸ*/ )
{
    *(double*)pMem = atof( pcCSV );
    return (double*)pMem + 1;
}



